'''
Module to manipulate a VASSAL module (VMOD)
'''
from . base import *
from . element import *
from . folder import *
from . globalkey import *
from . gameelements import *
from . mapelements import *
from . globalproperty import *
from . turn import *
from . documentation import *
from . player import *
from . widget import *
from . grid import *
from . zone import *
from . board import *
from . map import *
from . chart import *
from . command import *
from . trait import *
from . withtraits import *
from . extension import *
from . traits.area import *
from . traits.clone import *
from . traits.dynamicproperty import *
from . traits.globalproperty import *
from . traits.prototype import *
from . traits.place import *
from . traits.report import *
from . traits.calculatedproperty import *
from . traits.restrictcommand import *
from . traits.label import *
from . traits.layer import *
from . traits.globalcommand import *
from . traits.globalhotkey import *
from . traits.nostack import *
from . traits.deselect import *
from . traits.restrictaccess import *
from . traits.rotate import *
from . traits.stack import *
from . traits.mark import *
from . traits.mask import *
from . traits.trail import *
from . traits.delete import *
from . traits.sendto import *
from . traits.moved import *
from . traits.skel import *
from . traits.submenu import *
from . traits.basic import *
from . traits.trigger import *
from . traits.nonrect import *
from . traits.click import *
from . traits.mat import MatTrait
from . traits.cargo import CargoTrait
from . traits.movefixed import MoveFixedTrait
from . traits.sheet import SheetTrait
from . traits.hide import HideTrait
from . traits.retrn import ReturnTrait
from . game import *
from . buildfile import *
from . moduledata import *
from . save import *
from . vsav import *
from . vmod import *
from . exporter import *
from . merger import Merger
